from django.forms import CharField, ModelForm, Textarea, ValidationError

from core.models import Feedback


class FeedbackForm(ModelForm):
    text = CharField(
        widget=Textarea(
            attrs={
                'name': 'text',
                'id': 'id_text',
            }
        ),
        label='',
        required=False,
    )

    def clean(self):
        cleaned_data = super().clean()

        full_score = self.instance.of_submission.type.full_score
        if not cleaned_data.get("score") <= full_score:
            raise ValidationError(
                "Score too high. Maximum score is %(max_score)d",
                code='over_max_score',
                params={'max_score': full_score},
            )

        if not cleaned_data.get("text"):
            cleaned_data["status"] = Feedback.EDITABLE
            raise ValidationError(
                "Feedback should not be empty",
                code="is_empty",
            )

    class Meta:
        model = Feedback
        auto_id = False
        fields = ('text', 'score', 'status')
