
import hashlib

from django.contrib.auth.models import User
from rest_framework import serializers

from core.models import Feedback, Student, Submission, SubmissionType


class SubmissionTypeSerializer(serializers.ModelSerializer):

    def create(self, validated_data):
        return SubmissionType(**validated_data)

    class Meta:
        model = SubmissionType
        exclude = ('slug',)


class CreateStudentSerializer(serializers.ModelSerializer):

    username = serializers.CharField(source='user.username')
    email    = serializers.CharField(source='user.email')
    password = serializers.CharField(source='user.password')

    class Meta:
        model = Student
        fields = ('username', 'name', 'email', 'matrikel_no', 'password')
        extra_kwargs = {'password': {'write_only': True}}

    def to_representation(self, obj):
        return {
            'username'      : obj.user.username,
            'name'          : obj.name,
            'matrikel_no'   : obj.matrikel_no,
        }

    def create(self, validated_data):
        user = User(
            email=validated_data['email'],
            username=validated_data['username']
        )
        user.set_password(validated_data['password'])

        return Student.objects.create(
            name=validated_data['name'],
            matrikel_no=validated_data['matrikel_no'],
            user=user,
        )


class CreateSubmissionSerializer(serializers.ModelSerializer):

    type = serializers.SlugRelatedField(
        queryset=SubmissionType.objects.all(),
        slug_field='name'
    )

    student = serializers.SlugRelatedField(
        queryset=User.objects.all(),
        slug_field='username'
    )

    class Meta:
        model = Submission
        fields = ('type', 'text', 'pre_corrections', 'student')

    def create(self, validated_data):
        validated_data['student'] = validated_data['student'].student
        return Submission.objects.create(**validated_data)


class AnonymousFeedbackSerializer(serializers.ModelSerializer):

    def to_representation(self, obj):
        return {
            'feedback'  : obj.text,
            'score'     : obj.score,
            'tutor'     : obj.of_tutor.username,
            'student'   : hashlib.sha256(
                obj.of_submission.student.matrikel_no.encode() +
                obj.of_submission.student.name.encode()).hexdigest(),
            'code'      : obj.of_submission.text
        }

    def create(self, validated_data):
        return NotImplemented

    class Meta:
        model = Feedback
        fields = ()
