from django.utils.decorators import method_decorator
from django.views.generic import DetailView, ListView, View

from core.custom_annotations import group_required
from core.models import SubmissionType, get_annotated_tutor_list


class StudentView(View):

    @method_decorator(group_required('Students',))
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)


class StudentListView(StudentView, ListView):
    pass


class StudentDetailView(StudentView, DetailView):
    pass


class TutorView(View):

    @method_decorator(group_required('Tutors',))
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)


class TutorListView(TutorView, ListView):
    pass


class TutorDetailView(TutorView, DetailView):
    pass


class ReviewerView(View):

    @method_decorator(group_required('Reviewers',))
    def dispatch(self, *args, **kwargs):
        return super().dispatch(*args, **kwargs)


class ReviewerListView(ReviewerView, ListView):

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        return {
            'submission_type_list': SubmissionType.get_annotated_feedback_count(),
            'tutor_list':           get_annotated_tutor_list(),
            **context,
        }


class ReviewerDetailView(ReviewerView, DetailView):
    pass
