from django.views.generic import DetailView

from core.custom_annotations import in_groups
from core.models import Feedback, Student, Submission
from core.views.generics import (ReviewerDetailView, ReviewerListView,
                                 StudentView)


class StudentSubmissionView(StudentView, DetailView):

    template_name   = 'core/s/single_submission.html'
    model           = Submission

    def get_object(self):
        obj = Submission.objects.get(slug=self.kwargs['slug'])
        if in_groups(self.request.user, ('Students', )) and hasattr(obj, 'feedback') and obj.feedback.status == Feedback.ACCEPTED:
            obj.seen_by_student = True
            obj.save()
        return obj


class ReviewerSubmissionView(ReviewerDetailView):

    model         = Submission
    template_name = 'core/r/single_submission.html'

    def get_object(self):
        return Submission.objects.get(slug=self.kwargs['slug'])


class ReviewerSubmissionListView(ReviewerListView):

    model               = Submission
    template_name       = 'core/r/student_submission_list.html'
    context_object_name = 'submission_list'


class ReviewerStudentListView(ReviewerListView):

    model               = Student
    template_name       = 'core/r/student_list.html'
    context_object_name = 'student_list'

    def get_queryset(self):
        return self.model.get_overall_score_annotated_submission_list()
