from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.http import HttpResponseRedirect
from django.urls import reverse

from core.custom_annotations import in_groups
from core.models import Feedback, Student, SubmissionType
from core.views.generics import (ReviewerListView, StudentDetailView,
                                 TutorDetailView)


@login_required(login_url='/')
def user_home(request):
    if in_groups(request.user, ('Students', )):
        return StudentStartPage.as_view()(request)
    elif in_groups(request.user, ('Tutors', )):
        return TutorStartPage.as_view()(request)
    elif in_groups(request.user, ('Reviewers', )):
        return ReviewerFeedbackListView.as_view()(request)
    else:
        return HttpResponseRedirect(reverse('index'))


class TutorStartPage(TutorDetailView):

    model               = User
    template_name       = 'core/t/tutor_startpage.html'
    context_object_name = 'tutor'

    def get_object(self):
        return self.request.user

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        return {
            'submission_type_list': SubmissionType.get_annotated_feedback_count(),
            'feedback_open_list': Feedback.get_open_feedback(self.get_object()),
            **context
        }


class StudentStartPage(StudentDetailView):

    model               = Student
    template_name       = 'core/s/student_startpage.html'

    def get_object(self):
        return self.request.user.student


class ReviewerFeedbackListView(ReviewerListView):
    """ This is the de facto startpage of the reviewer accounts"""

    model         = Feedback
    template_name = 'core/r/reviewer_startpage.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        return {
            'feedback_list_manual': self.model.objects.filter(origin=Feedback.MANUAL),
            'feedback_list_empty': self.model.objects.filter(origin=Feedback.WAS_EMPTY),
            'feedback_list_did_not_compile': self.model.objects.filter(origin=Feedback.DID_NOT_COMPILE),
            'feedback_list_could_not_link': self.model.objects.filter(origin=Feedback.COULD_NOT_LINK),
            **context
        }
