#!/usr/local/bin/python3
"""This script adds compiler output to the json output of the convert script

[description]
"""
import subprocess
import json
import os

TEMP_DIR        = 'temp_code'
OUTFILE         = 'submissions_compiled.json'
INFILE          = 'submissions.json'
OBJECT_DIR      = 'klausur_tag_01/objects'

ALL_OK          = 0
COMPILE_ERROR   = 1
LINKER_ERROR    = 2
WARNINGS        = 3


def get_compiler_output(task_id, text):
    dst = open(os.path.join(TEMP_DIR, task_id + '.o'), 'w')
    try:
        compile_cmd = subprocess.run(
            [
                "gcc-6", "-Wall", "-std=c11", "-c", "-xc",
                "-o", dst.name,
                f"-I{TEMP_DIR}",
                "-"
            ],
            stderr=subprocess.PIPE,
            input=text,
            encoding='utf-8',
        )

        if compile_cmd.returncode:
            return compile_cmd, None, COMPILE_ERROR # it fucking failed

        object_code = os.path.join(OBJECT_DIR, f"{task_id}-testing.o")
        if os.path.exists(object_code): # this is ok
            link_cmd = subprocess.run(
                [
                    "gcc-6",
                    "-o", "/dev/null",
                    dst.name,
                    object_code

                ],
                stderr=subprocess.PIPE,
                encoding='utf-8',
            )
        else: # this case is weird
            if task_id == 'a05': # for day 2 task a05 its ok to just compile
                return compile_cmd, None, compile_cmd.returncode
            elif task_id == 'a06':
                link_cmd = subprocess.run(
                    [
                        "gcc-6",
                        "-o", "/dev/null",
                        dst.name,
                        os.path.join(TEMP_DIR, "a05.o"),
                    ],
                    stderr=subprocess.PIPE,
                    encoding='utf-8',
                )

        if link_cmd.returncode:
            return compile_cmd, link_cmd, LINKER_ERROR

        return compile_cmd, link_cmd, ALL_OK
    finally:
        dst.close()


def main():
    with open(INFILE, 'r', encoding='utf-8') as submissions:
        data = json.JSONDecoder().decode(submissions.read())

    total = len(data)
    for i, (username, userinfo) in enumerate(data.items()):
        print(f"\r- {i+1}/{total} done. processing submissions of {username}\t\t\t\t", end='')

        # create new entry
        co = userinfo['compiler_output'] = {}

        for task_id, submission in userinfo['submissions'].items():
            if submission:
                _compile, _link, err = get_compiler_output(task_id, submission)
                msg = {
                    COMPILE_ERROR: "\n\n[FAIL] DID NOT COMPILE",
                    LINKER_ERROR: "\n\n[FAIL] COULD NOT LINK",
                    ALL_OK: "",
                }[err]
                co[task_id] = _compile.stderr + (_link.stderr if _link else "") + msg
            else:
                co[task_id] = ""

    print()

    with open(OUTFILE, 'w', encoding='utf-8') as submissions:
        submissions.write(json.JSONEncoder().encode(data))

if __name__ == '__main__':
    main()
